@@ -38,6 +38,32 @@ namespace Clasp { namespace Asp {
 /////////////////////////////////////////////////////////////////////////////////////////
 // class LpStats
 /////////////////////////////////////////////////////////////////////////////////////////
+#define LP_STATS( APPLY )                                      \
+	APPLY("bodies"              , bodies)                        \
+	APPLY("atoms"               , atoms)                         \
+	APPLY("atoms_aux"           , auxAtoms)                      \
+	APPLY("sccs"                , sccs)                          \
+	APPLY("sccs_non_hcf"        , nonHcfs)                       \
+	APPLY("gammas"              , gammas)                        \
+	APPLY("ufs_nodes"           , ufsNodes)                      \
+	APPLY("rules"               , rules())                       \
+	APPLY("rules_basic"         , rules(BASICRULE).first)        \
+	APPLY("rules_choice"        , rules(CHOICERULE).first)       \
+	APPLY("rules_constraint"    , rules(CONSTRAINTRULE).first)   \
+	APPLY("rules_weight"        , rules(WEIGHTRULE).first)       \
+	APPLY("rules_disjunctive"   , rules(DISJUNCTIVERULE).first)  \
+	APPLY("rules_optimize"      , rules(OPTIMIZERULE).first)     \
+	APPLY("rules_tr_basic"      , rules(BASICRULE).second)       \
+	APPLY("rules_tr_choice"     , rules(CHOICERULE).second)      \
+	APPLY("rules_tr_constraint" , rules(CONSTRAINTRULE).second)  \
+	APPLY("rules_tr_weight"     , rules(WEIGHTRULE).second)      \
+	APPLY("rules_tr_disjunctive", rules(DISJUNCTIVERULE).second) \
+	APPLY("rules_tr_optimize"   , rules(OPTIMIZERULE).second)    \
+	APPLY("eqs"                 , eqs())                         \
+	APPLY("eqs_atom"            , eqs(Var_t::atom_var))          \
+	APPLY("eqs_body"            , eqs(Var_t::body_var))          \
+	APPLY("eqs_other"           , eqs(Var_t::atom_body_var))
+
 void LpStats::reset() {
 	std::memset(this, 0, sizeof(LpStats));
 }
@@ -67,45 +93,22 @@ void LpStats::accu(const LpStats& o) {
 		rules_[i].second += o.rules_[i].second;
 	}
 }
+
 double LpStats::operator[](const char* key) const {
-#define RETURN_IF(x) if (std::strcmp(key, #x) == 0) return double(x)
-#define MAP_IF(x, A) if (std::strcmp(key, x) == 0)  return double( (A) )
-	RETURN_IF(bodies);
-	RETURN_IF(atoms);
-	RETURN_IF(auxAtoms);
-	RETURN_IF(sccs);
-	RETURN_IF(nonHcfs);
-	RETURN_IF(gammas);
-	RETURN_IF(ufsNodes);
-	MAP_IF("rules"             , rules());
-	MAP_IF("basicRules"        , rules(BASICRULE).first);
-	MAP_IF("choiceRules"       , rules(CHOICERULE).first);
-	MAP_IF("constraintRules"   , rules(CONSTRAINTRULE).first);
-	MAP_IF("weightRules"       , rules(WEIGHTRULE).first);
-	MAP_IF("disjunctiveRules"  , rules(DISJUNCTIVERULE).first);
-	MAP_IF("optimizeRules"     , rules(OPTIMIZERULE).first);
-	MAP_IF("basicRulesTr"      , rules(BASICRULE).second);
-	MAP_IF("choiceRulesTr"     , rules(CHOICERULE).second);
-	MAP_IF("constraintRulesTr" , rules(CONSTRAINTRULE).second);
-	MAP_IF("weightRulesTr"     , rules(WEIGHTRULE).second);
-	MAP_IF("disjunctiveRulesTr", rules(DISJUNCTIVERULE).second);	
-	MAP_IF("optimizeRulesTr"   , rules(OPTIMIZERULE).second);
-	MAP_IF("eqs"               , eqs());
-	MAP_IF("atomEqs"           , eqs(Var_t::atom_var));
-	MAP_IF("bodyEqs"           , eqs(Var_t::body_var));
-	MAP_IF("otherEqs"          , eqs(Var_t::atom_body_var));
-#undef RETURN_IF
+#define MAP_IF(x, A) if (std::strcmp(key, x) == 0)  return double( (A) );
+	LP_STATS(MAP_IF)
+#undef MAP_IF
 	return -1.0;
 }
 const char* LpStats::keys(const char* path) {
 	if (!path || !*path) {
-		return "bodies\0atoms\0auxAtoms\0sccs\0nonHcfs\0gammas\0ufsNodes\0rules\0"
-		       "basicRules\0choiceRules\0constraintRules\0weightRules\0disjunctiveRules\0optimizeRules\0"
-		       "basicRulesTr\0choiceRulesTr\0constraintRulesTr\0weightRulesTr\0disjunctiveRulesTr\0optimizeRulesTr\0"
-		       "eqs\0atomEqs\0bodyEqs\0otherEqs\0";
+#define KEY(x, y) x "\0"
+		return LP_STATS(KEY);
+#undef KEY
 	}
 	return 0;
 }
+#undef LP_STATS
 /////////////////////////////////////////////////////////////////////////////////////////
 // class LogicProgram
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -129,9 +132,9 @@ bool toConstraint(NT* node, const LogicProgram& prg, ClauseCreator& c) {
 	return !node->relevant() || node->addConstraints(prg, c);
 }
 }
-LogicProgram::LogicProgram() : nonHcfCfg_(0), minimize_(0), incData_(0) { accu = 0; }
+LogicProgram::LogicProgram() : nonHcfCfg_(0), minimize_(0), incData_(0), startAux_(0) { accu = 0; }
 LogicProgram::~LogicProgram() { dispose(true); }
-LogicProgram::Incremental::Incremental() : startAtom(1), startAux(1), startScc(0) {}
+LogicProgram::Incremental::Incremental() : startAtom(1), startScc(0) {}
 void LogicProgram::dispose(bool force) {
 	// remove rules
 	std::for_each( bodies_.begin(), bodies_.end(), DestroyObject() );
@@ -154,60 +157,17 @@ void LogicProgram::dispose(bool force) {
 	extended_.clear();
 	VarVec().swap(initialSupp_);
 	rule_.clear();
-	if (force) {
+	if (force || !incData_) {
 		std::for_each( atoms_.begin(), atoms_.end(), DeleteObject() );
 		AtomList().swap(atoms_);
 		delete incData_;
 		VarVec().swap(propQ_);
 		ruleState_.clearAll();
-	}
-	else if (incData_) {
-		// clean up atoms
-		// reset prop queue
-		propQ_.assign(1, getFalseId());
-		uint32 startAux = incData_->startAux;
-		assert(startAux <= atoms_.size());
-		// remove rule associations
-		for (VarVec::size_type i = 1; i != startAux; ++i) {
-			PrgAtom* a = atoms_[i];
-			// remove any dangling references
-			a->clearSupports();
-			a->clearDeps(PrgAtom::dep_all);
-			a->setIgnoreScc(false);
-			if (a->eq() && getEqAtom(i) >= startAux) {
-				// atom i is equivalent to some aux atom 
-				// make i the new root
-				PrgAtom* eq = atoms_[getEqAtom(i)];
-				assert(!eq->eq());
-				eq->setEq(i);
-				a->resetId(i, false);
-				a->setLiteral(eq->literal());
-			}
-			if (a->relevant() || a->frozen()) {
-				ValueRep v = a->value();
-				a->setValue(value_free);
-				a->resetId(i, !a->frozen());
-				if (ctx()->master()->value(a->var()) != value_free && !a->frozen()) {
-					v = ctx()->master()->isTrue(a->literal()) ? value_true : value_false;
-				}
-				if (v != value_free) { assignValue(a, v); }
-			}
-			else if (!a->eq() && a->value() == value_false) {
-				a->setEq(getFalseId());
-			}
-		}
-		// delete any introduced aux atoms
-		// this is safe because aux atoms are never part of the input program
-		// it is necessary in order to free their ids, i.e. the id of an aux atom
-		// from step I might be needed for a program atom in step I+1
-		for (VarVec::size_type i = startAux; i != atoms_.size(); ++i) {
-			delete atoms_[i];
-		}
-		atoms_.erase(atoms_.begin()+startAux, atoms_.end());
+		stats.reset();
+		startAux_ = atoms_.size();
 	}
 	activeHead_.clear();
 	activeBody_.reset();
-	stats.reset();
 }
 bool LogicProgram::doStartProgram() {
 	dispose(true);
@@ -218,40 +178,75 @@ bool LogicProgram::doStartProgram() {
 	nonHcfCfg_= 0;
 	incData_  = 0;
 	ctx()->symbolTable().clear();
-	ctx()->symbolTable().startInit();
+	ctx()->symbolTable().startInit(SymbolTable::map_indirect);
 	return true;
 }
 void LogicProgram::setOptions(const AspOptions& opts) {
 	opts_ = opts;
-	if (opts_.suppMod) {
-		if (opts_.iters != 5 && ctx()) { ctx()->report(warning(Event::subsystem_prepare, "'supp-models' implies 'eq=0'")); }
-		opts_.noEq();
-		opts_.noScc();
+	if (opts.suppMod) {
+		if (!incData_ || incData_->startScc == 0) {
+			if (opts_.iters != 5 && ctx()) { ctx()->report(warning(Event::subsystem_prepare, "'supp-models' implies 'eq=0'.")); }
+			opts_.noEq();
+			opts_.noScc();
+		}
+		else {
+			if (ctx()) { ctx()->report(warning(Event::subsystem_facade, "'supp-models' ignored for non-tight programs.")); }
+			opts_.suppMod = 0;
+			opts_.noSCC   = 0;
+		}
 	}
 }
 bool LogicProgram::doParse(StreamSource& prg) { return DefaultLparseParser(*this).parse(prg); }
 bool LogicProgram::doUpdateProgram() {
-	CLASP_ASSERT_CONTRACT(frozen() || !incData_);
-	if (!incData_)  { incData_ = new Incremental(); }
-	if (!frozen())  { return true; }
+	if (!incData_) { incData_ = new Incremental(); ctx()->symbolTable().incremental(true); }
+	if (!frozen()) { return true; }
 	// delete bodies/disjunctions...
 	dispose(false);
 	setFrozen(false);
-	incData_->startAtom = (uint32)atoms_.size();
-	incData_->startAux  = (uint32)atoms_.size();
 	incData_->update.clear();
 	incData_->frozen.swap(incData_->update);
-	ctx()->symbolTable().startInit();
-	// add supported atoms from previous steps 
+	ctx()->symbolTable().startInit(SymbolTable::map_indirect);
+	// reset prop queue and add supported atoms from previous steps 
 	// {ai | ai in P}.
-	PrgBody* support = incData_->startAux > 1 ? getBodyFor(activeBody_) : 0;
-	for (VarVec::size_type i = 1; i != incData_->startAux; ++i) {
-		PrgAtom* a = atoms_[i];
-		if (a->relevant() && !a->frozen() && a->value() != value_false) {
-			a->setIgnoreScc(true);
-			support->addHead(a, PrgEdge::CHOICE_EDGE);
+	PrgBody* support = startAux_ > 1 ? getBodyFor(activeBody_) : 0;
+	propQ_.assign(1, getFalseId());
+	for (Var i = 1, end = startAux_; i != end; ++i) {
+		if (getEqAtom(i) >= startAux_) {
+			// atom is equivalent to some aux atom - make i the new root
+			uint32 r = getEqAtom(i);
+			std::swap(atoms_[i], atoms_[r]);
+			atoms_[r]->setEq(i);
 		}
+		// remove dangling references
+		PrgAtom* a = atoms_[i];
+		a->clearSupports();
+		a->clearDeps(PrgAtom::dep_all);
+		a->setIgnoreScc(false);
+		if (a->relevant() || a->frozen()) {
+			ValueRep v = a->value();
+			a->setValue(value_free);
+			a->resetId(i, !a->frozen());
+			if (ctx()->master()->value(a->var()) != value_free && !a->frozen()) {
+				v = ctx()->master()->isTrue(a->literal()) ? value_true : value_false;
+			}
+			if (v != value_free) { assignValue(a, v); }
+			if (!a->frozen() && a->value() != value_false) {
+				a->setIgnoreScc(true);
+				support->addHead(a, PrgEdge::CHOICE_EDGE);
+			}
+		}
+		else if (!a->eq() && a->value() == value_false) {
+			a->setEq(getFalseId());
+		} 
 	}
+	// delete any introduced aux atoms
+	// this is safe because aux atoms are never part of the input program
+	// it is necessary in order to free their ids, i.e. the id of an aux atom
+	// from step I might be needed for a program atom in step I+1
+	std::for_each(atoms_.begin()+startAux_, atoms_.end(), DeleteObject());
+	atoms_.erase(atoms_.begin()+startAux_, atoms_.end());
+	incData_->startAtom = startAux_ = (uint32)atoms_.size();
+	stats.reset();
 	return true;
 }
 bool LogicProgram::doEndProgram() {
@@ -402,7 +397,7 @@ Var LogicProgram::newAtom() {
 
 LogicProgram& LogicProgram::setAtomName(Var atomId, const char* name) {
 	check_not_frozen();
-	check_modular(atomId >= startAtom(), atomId);
+	check_modular(isNew(atomId), atomId);
 	resize(atomId);
 	ctx()->symbolTable().addUnique(atomId, name);
 	return *this;
@@ -412,34 +407,36 @@ LogicProgram& LogicProgram::setCompute(Var atomId, bool pos) {
 	resize(atomId);
 	ValueRep v = pos ? value_weak_true : value_false;
 	PrgAtom* a = atoms_[atomId];
-	assert(!a->hasVar() || a->state() != PrgAtom::state_normal);
+	assert(!a->hasVar() || a->frozen());
 	assignValue(a, v);
 	return *this;
 }
 
+PrgAtom* LogicProgram::setExternal(Var atomId, ValueRep v) {
+	PrgAtom* a = resize(atomId);
+	if (a->frozen() || (isNew(atomId) && !a->supports())) {
+		if (!incData_)    { incData_ = new Incremental(); }
+		if (!a->frozen()) { incData_->update.push_back(a->id()); }
+		a->markFrozen(v);
+		return a;
+	}
+	return 0; // atom is defined or from a previous step - ignore!
+}
+
 LogicProgram& LogicProgram::freeze(Var atomId, ValueRep value) {
 	check_not_frozen();
-	CLASP_ASSERT_CONTRACT_MSG(incData_, "LogicProgram::updateProgram() not called!");
-	assert(incData_->frozen.empty());
-	PrgAtom* a = resize(atomId);
-	if (!a->inFlux() && (a->frozen() || (atomId >= startAtom() && !a->supports()))) {
-		CLASP_ASSERT_CONTRACT(value == value_false || value == value_true);
-		if (!a->frozen()) { incData_->update.push_back(atomId); }
-		a->setState(value == value_false ? PrgAtom::state_freeze : PrgAtom::state_freeze_true);
-	}
-	// else: atom is defined or from a previous step - ignore!
+	CLASP_ASSERT_CONTRACT(value < value_weak_true);
+	setExternal(atomId, value);
 	return *this;
 }
 
 LogicProgram& LogicProgram::unfreeze(Var atomId) {
 	check_not_frozen();
-	CLASP_ASSERT_CONTRACT_MSG(incData_, "LogicProgram::updateProgram() not called!");
-	PrgAtom* a = resize(atomId);
-	if (!a->inFlux() && (atomId >= startAtom() || a->frozen())) {
-		if (!a->frozen()) { incData_->update.push_back(atomId); }
-		a->setState(PrgAtom::state_in_flux);
+	PrgAtom* a = setExternal(atomId, value_free);
+	if (a && a->supports() == 0) {
+		// add dummy edge - will be removed once we update the set of frozen atoms
+		a->addSupport(PrgEdge::noEdge());
 	}
-	// else: atom is from a previous step - ignore!
 	return *this;
 }
 
@@ -486,7 +483,8 @@ Literal LogicProgram::getLiteral(Var atomId) const {
 void LogicProgram::doGetAssumptions(LitVec& out) const {
 	if (incData_) {
 		for (VarVec::const_iterator it = incData_->frozen.begin(), end = incData_->frozen.end(); it != end; ++it) {
-			out.push_back( getAtom(getEqAtom(*it))->assumption() );
+			Literal lit = getAtom(getEqAtom(*it))->assumption();
+			if (lit != posLit(0)) { out.push_back( lit ); }
 		}
 	}
 }
@@ -504,8 +502,7 @@ void LogicProgram::addRuleImpl(RuleType r, const VarVec& heads, BodyInfo& body)
 			bool ignoreScc  = opts_.noSCC || b->size() == 0;
 			for (VarVec::const_iterator it = heads.begin(), end = heads.end(); it != end; ++it) {
 				PrgAtom* a = resize(*it);
-				if (a->frozen() && a->supports() == 0) { unfreeze(*it); }
-				check_modular(*it >= startAtom() || a->inFlux() || a->value() == value_false, *it);
+				check_modular(isNew(*it) || a->frozen() || a->value() == value_false, *it);
 				if (r != DISJUNCTIVERULE) {
 					// Note: b->heads may now contain duplicates. They are removed in PrgBody::simplifyHeads.
 					b->addHead(a, t);
@@ -577,12 +574,9 @@ bool LogicProgram::transformNoAux(RuleType r, const BodyInfo& body) const {
 }
 
 void LogicProgram::transformExtended() {
-	uint32 a   = numAtoms();
-	if (incData_) {
-		// remember starting position of aux atoms so
-		// that we can remove them on next incremental step
-		incData_->startAux = (uint32)atoms_.size();
-	}
+	// remember starting position of aux atoms so
+	// that we can remove them on next incremental step
+	uint32 a  = numAtoms();
 	RuleTransform tm;
 	for (RuleList::size_type i = 0; i != extended_.size(); ++i) {
 		incTr(extended_[i]->type(), tm.transform(*this, *extended_[i]));
@@ -592,7 +586,7 @@ void LogicProgram::transformExtended() {
 	incTrAux(numAtoms() - a);
 }
 
-void LogicProgram::transformIntegrity(uint32 maxAux) {
+void LogicProgram::transformIntegrity(uint32 nAtoms, uint32 maxAux) {
 	if (stats.rules(CONSTRAINTRULE).second == 0) { return; }
 	// find all constraint rules that are integrity constraints
 	BodyList integrity;
@@ -602,13 +596,13 @@ void LogicProgram::transformIntegrity(uint32 maxAux) {
 			integrity.push_back(b);
 		}
 	}
-	if (!integrity.empty() && (integrity.size() == 1 || (atoms_.size()/double(bodies_.size()) > 0.5 && integrity.size() / double(bodies_.size()) < 0.01))) {
-		uint32 A      = static_cast<uint32>(atoms_.size());
+	if (!integrity.empty() && (integrity.size() == 1 || (nAtoms/double(bodies_.size()) > 0.5 && integrity.size() / double(bodies_.size()) < 0.01))) {
+		uint32 aux = static_cast<uint32>(atoms_.size());
 		// transform integrity constraints
 		for (BodyList::size_type i = 0; i != integrity.size(); ++i) {
 			PrgBody* b = integrity[i];
 			uint32 est = b->bound()*( b->sumW()-b->bound() );
-			if (est > maxAux) { 
+			if (est > maxAux) {
 				// reached limit on aux atoms - stop transformation
 				break; 
 			}
@@ -631,7 +625,7 @@ void LogicProgram::transformIntegrity(uint32 maxAux) {
 			b->markRemoved();
 		}
 		// create vars for new atoms/bodies
-		for (uint32 i = A; i != atoms_.size(); ++i) {
+		for (uint32 i = aux; i != atoms_.size(); ++i) {
 			PrgAtom* a = atoms_[i];
 			for (PrgAtom::sup_iterator it = a->supps_begin(); it != a->supps_end(); ++it) {
 				PrgBody* nb = bodies_[it->node()];
@@ -663,19 +657,21 @@ void LogicProgram::updateFrozenAtoms() {
 	for (VarVec::iterator it = j, end = incData_->update.end(); it != end; ++it) {
 		Var id     = getEqAtom(*it);
  		PrgAtom* a = getAtom(id);
-		if (a->inFlux()) {
-			assert(a->id() == id && a->scc() == PrgNode::noScc && !a->inUpper());
-			a->setState(PrgAtom::state_normal);
-			if (id < startAtom()) {
-				// unfreeze previously frozen atom
-				a->markSeen(false);
-				a->markDirty();
-				*j++ = id; // keep in list so that we can later perform completion
+		assert(a->frozen());
+		a->resetId(id, false);
+		if (a->supports() != 0) {
+			a->clearFrozen();
+			if (*a->supps_begin() == PrgEdge::noEdge()) {
+				// remove dummy edge added in unfreeze()
+				a->removeSupport(PrgEdge::noEdge());
+			}
+			if (!isNew(id)) {
+				// keep in list so that we can later perform completion
+				*j++ = id; 
 			}
 		}
-		else if (a->frozen()) {
+		else {
 			assert(a->relevant() && a->supports() == 0);
-			a->resetId(id, false);
 			if (!support) { support = getBodyFor(activeBody_); }
  			a->setIgnoreScc(true);
  			support->addHead(a, PrgEdge::CHOICE_EDGE);
@@ -687,10 +683,10 @@ void LogicProgram::updateFrozenAtoms() {
 
 void LogicProgram::prepareProgram(bool checkSccs) {
 	assert(!frozen());
+	startAux_ = (uint32)atoms_.size();
 	transformExtended();
-	if (opts_.normalize) { /* normalize(); */ assert(false);  }
-	stats.atoms = numAtoms() - (startAtom()-1);
 	stats.bodies= numBodies();
+	if (opts_.normalize) { /* normalize(); */ assert(false);  }
 	updateFrozenAtoms();
 	setFrozen(true);
 	Preprocessor p;
@@ -699,7 +695,8 @@ void LogicProgram::prepareProgram(bool checkSccs) {
 		return;
 	}
 	if (opts_.erMode == mode_transform_integ || opts_.erMode == mode_transform_dynamic) {
-		transformIntegrity(std::min(uint32(15000), uint32(numAtoms())<<1));
+		uint32 nAtoms = startAux_ - startAtom();
+		transformIntegrity(nAtoms, std::min(uint32(15000), nAtoms*2));
 	}
 	addMinimize();
 	uint32 sccs = 0;
@@ -722,7 +719,7 @@ void LogicProgram::prepareProgram(bool checkSccs) {
 	else { stats.sccs = PrgNode::noScc; }
 	finalizeDisjunctions(p, sccs);
 	prepareComponents();
-	stats.atoms = numAtoms() - (startAtom()-1);
+	stats.atoms = static_cast<uint32>(atoms_.size()) - startAtom();
 	bodyIndex_.clear();
 	disjIndex_.clear();
 }
@@ -794,7 +791,7 @@ void LogicProgram::finalizeDisjunctions(Preprocessor& p, uint32 numSccs) {
 					B->addHead(x, PrgEdge::NORMAL_EDGE);
 					x->assignVar(*this, *x->supps_begin());
 					x->setInUpper(true);
-					x->markSeen(true);
+					x->setSeen(true);
 					++added;
 					if ((sccMap[scc] & is_scc_non_hcf) == 0) {
 						sccMap[scc] |= is_scc_non_hcf;
@@ -852,7 +849,7 @@ void LogicProgram::prepareComponents() {
 				PrgAtom* a = self->getAtom(x);
 				self->sccAtoms_.push_back(a);
 				a->setScc(scc);
-				a->markSeen(true);
+				a->setSeen(true);
 				atoms.push_back(x);
 				return x;
 			}
@@ -870,7 +867,6 @@ void LogicProgram::prepareComponents() {
 		} tr(this);
 		RuleTransform trans;
 		setFrozen(false);
-		if (incData_) { incData_->startAux = (uint32)atoms_.size(); }
 		for (BodyList::const_iterator it = ext.begin(), end = ext.end(); it != end; ++it) {
 			uint32 scc = (*it)->scc(*this);
 			rule_.clear();
@@ -1148,7 +1144,7 @@ Literal LogicProgram::getEqAtomLit(Literal lit, const BodyList& supports, Prepro
 	PrgAtom* aux = getAtom(auxV);
 	uint32 scc   = PrgNode::noScc;
 	aux->setLiteral(lit);
-	aux->markSeen(true);
+	aux->setSeen(true);
 	p.setRootAtom(aux->literal(), auxV);
 	for (BodyList::const_iterator sIt = supports.begin(); sIt != supports.end(); ++sIt) {
 		PrgBody* b = *sIt;
@@ -1208,7 +1204,7 @@ PrgBody* LogicProgram::assignBodyFor(BodyInfo& body, EdgeType depEdge, bool simp
 			b = bodies_[eqId];
 		}
 	}
-	b->markSeen(true);
+	b->setSeen(true);
 	b->assignVar(*this);
 	return b;
 }
@@ -1309,10 +1305,9 @@ uint32 LogicProgram::removeBody(PrgBody* b, uint32 hash) {
 PrgAtom* LogicProgram::mergeEqAtoms(PrgAtom* a, Var rootId) {
 	rootId        = getEqAtom(rootId);
 	PrgAtom* root = getAtom(rootId);
-	ValueRep mv   = getMergeValue(a, root);
-	assert(!a->eq() && !root->eq());
+	ValueRep mv   = getMergeValue(a, root);	
+	assert(!a->eq() && !root->eq() && !a->frozen());
 	if (a->ignoreScc()) { root->setIgnoreScc(true); }
-	if (a->frozen())    { root->setState(std::max(a->state(), root->state())); }
 	if (mv != a->value()    && !assignValue(a, mv))   { return 0; }
 	if (mv != root->value() && !assignValue(root, mv)){ return 0; }
 	a->setEq(rootId);
