@@ -506,7 +506,7 @@ PrgNode::PrgNode(uint32 id, bool checkScc)
 /////////////////////////////////////////////////////////////////////////////////////////
 PrgHead::PrgHead(uint32 id, NodeType t, uint32 data, bool checkScc)
 	: PrgNode(id, checkScc)
-	, data_(data), upper_(0), dirty_(0), state_(0), isAtom_(t == PrgEdge::ATOM_NODE)  {
+	, data_(data), upper_(0), dirty_(0), freeze_(0), isAtom_(t == PrgEdge::ATOM_NODE)  {
 	struct X { uint64 x; EdgeVec y; uint32 z; };
 	static_assert(sizeof(PrgHead) == sizeof(X), "Unsupported Alignment");
 }
@@ -544,7 +544,7 @@ bool PrgHead::simplifySupports(LogicProgram& prg, bool strong, uint32* numDiffSu
 		for (it = supports_.begin(); it != supports_.end(); ++it) {
 			PrgNode* x = prg.getSupp(*it);
 			if (x->relevant() && x->value() != value_false && (!strong || x->hasVar())) {
-				if      (!x->seen()) { *j++ = *it; x->markSeen(true); }
+				if      (!x->seen()) { *j++ = *it; x->setSeen(true); }
 				else if (!choices)   { continue; }
 				else                 {
 					for (n = supports_.begin(); n != it && n->node() != it->node(); ++n) {;}
@@ -563,7 +563,7 @@ bool PrgHead::simplifySupports(LogicProgram& prg, bool strong, uint32* numDiffSu
 		choices    = 0;
 		for (it = supports_.begin(); it != supports_.end(); ++it) {
 			PrgNode* x = prg.getSupp(*it);
-			x->markSeen(false);
+			x->setSeen(false);
 			if (strong && ctx.marked(x->literal())) { ctx.unmark(x->var()); }
 			if (it->isChoice()) {
 				++choices;
@@ -1341,7 +1341,6 @@ bool PrgBody::addConstraints(const LogicProgram& prg, ClauseCreator& gc) {
 		Literal negB= ~literal();
 		gc.start().add(literal()); 
 		for (const Literal* it = goals_begin(), *end = goals_end(); it != end; ++it) {
-			assert(it->var() != 0);
 			Literal li = prg.getAtom(it->var())->literal() ^ it->sign();
 			if (li == literal()) { taut = true; continue; }
 			if (!prg.ctx()->addBinary(negB, li)) { // [~B li]
