@@ -129,7 +129,9 @@ class LogicProgramTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testSimpleIncremental);
 	CPPUNIT_TEST(testIncrementalFreeze);
 	CPPUNIT_TEST(testIncrementalFreezeValue);
+	CPPUNIT_TEST(testIncrementalFreezeOpen);
 	CPPUNIT_TEST(testIncrementalKeepFrozen);
+	CPPUNIT_TEST(testIncrementalFreezeCompute);
 	CPPUNIT_TEST(testIncrementalUnfreezeUnsupp);
 	CPPUNIT_TEST(testIncrementalUnfreezeCompute);
 	CPPUNIT_TEST(testIncrementalCompute);
@@ -775,7 +777,7 @@ public:
 
 	void testAddClauses() {
 		ClauseObserver* o = new ClauseObserver;
-		ctx.master()->heuristic().reset(o);
+		ctx.master()->setHeuristic(o);
 		builder.start(ctx)
 			.setAtomName(1, "a").setAtomName(2, "b").setAtomName(3, "c")
 			.startRule().addHead(1).addToBody(2, false).endRule()								// a :- not b.
@@ -1518,6 +1520,36 @@ public:
 		CPPUNIT_ASSERT(solver.isFalse(ctx.symbolTable()[2].lit));
 		CPPUNIT_ASSERT(solver.isTrue(ctx.symbolTable()[3].lit));
 	}
+
+	void testIncrementalFreezeOpen() {
+		builder.start(ctx, LogicProgram::AspOptions().noEq());
+		// I1:
+		// freeze(x, value_free)
+		builder.updateProgram();
+		builder.setAtomName(1, "x").freeze(1, value_free);
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
+		LitVec assume;
+		builder.getAssumptions(assume);
+		Solver& solver = *ctx.master();
+		CPPUNIT_ASSERT(assume.empty());
+		CPPUNIT_ASSERT(solver.value(ctx.symbolTable()[1].lit.var()) == value_free);
+		
+		// I2: 
+		builder.updateProgram();
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
+		assume.clear();
+		builder.getAssumptions(assume);
+		CPPUNIT_ASSERT(assume.empty());
+		CPPUNIT_ASSERT(solver.value(ctx.symbolTable()[1].lit.var()) == value_free);
+
+		// I3:
+		builder.updateProgram();
+		builder.freeze(1, value_true);
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
+		assume.clear();
+		builder.getAssumptions(assume);
+		CPPUNIT_ASSERT(assume.size() == 1 && assume[0] == ctx.symbolTable()[1].lit);
+	}
 	void testIncrementalKeepFrozen() {
 		builder.start(ctx);
 		// I0:
@@ -1535,6 +1567,29 @@ public:
 		CPPUNIT_ASSERT(x->literal() == xLit);
 		CPPUNIT_ASSERT(x->frozen());
 	}
+	void testIncrementalFreezeCompute() {
+		builder.start(ctx);
+		// I0:
+		// freeze{x}.
+		builder.updateProgram();
+		builder.setAtomName(1, "x").setAtomName(2, "y").setAtomName(3, "z").setAtomName(4, "z2")
+			.freeze(1).freeze(2).freeze(3).freeze(4)
+			.setCompute(1, true).setCompute(2, true).setCompute(3, false).setCompute(4, false)
+		;
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram());
+		PrgAtom* x = builder.getAtom(1);
+		PrgAtom* y = builder.getAtom(2);
+		CPPUNIT_ASSERT(x->frozen() && y->frozen());
+		CPPUNIT_ASSERT(x->literal() != y->literal());
+		PrgAtom* z  = builder.getAtom(3);
+		PrgAtom* z2 = builder.getAtom(4);
+		CPPUNIT_ASSERT(z->frozen() && z2->frozen());
+		CPPUNIT_ASSERT(z->literal() == z2->literal());
+		// I1:
+		builder.updateProgram();
+		builder.setCompute(1, false);
+		CPPUNIT_ASSERT_EQUAL(false, builder.endProgram());
+	}
 	void testIncrementalUnfreezeUnsupp() {
 		builder.start(ctx, LogicProgram::AspOptions().noEq());
 		// I1:
