@@ -12,8 +12,7 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this file; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 //
 // NOTE: ProgramOptions is inspired by Boost.Program_options
@@ -26,7 +25,6 @@
 #endif
 #include <program_opts/program_options.h>
 #include <program_opts/errors.h>
-#include <program_opts/value_parser.h>
 #include <cassert>
 #include <cstring>
 #include <climits>
@@ -119,7 +117,7 @@ void FileWriter::write(const std::vector<char>& buf, std::size_t n) {
 // class Value
 ///////////////////////////////////////////////////////////////////////////////
 Value::Value(byte_t flagSet, State initial)
-	: state_(initial)
+	: state_(static_cast<byte_t>(initial))
 	, flags_(flagSet)
 	, descFlag_(0)
 	, optAlias_(0) {
@@ -146,7 +144,7 @@ Value* Value::desc(DescType t, const char* n) {
 	}
 	if (descFlag_ == 0 || descFlag_ == t) {
 		desc_.value = n;
-		descFlag_   = t;
+		descFlag_   = static_cast<byte_t>(t);
 		return this;
 	}
 	if (descFlag_ != desc_pack) {
@@ -177,10 +175,9 @@ const char* Value::implicit() const {
 
 bool Value::parse(const std::string& name, const std::string& value, State st) {
 	if (!value.empty() || !isImplicit()) return state(doParse(name, value), st);
-	const char*    x = implicit();
+	const char* x = implicit();
 	assert(x);
-	const FlagStr* y = FlagStr::find(x);
-	return state(doParse(name, y ? y->str:x), st);
+	return state(doParse(name, x), st);
 }
 ///////////////////////////////////////////////////////////////////////////////
 // class Option
@@ -238,6 +235,9 @@ void OptionGroup::addOption(std::auto_ptr<Option> option) {
 	SharedOptPtr opt(option.release());
 	options_.push_back(opt);
 }
+void OptionGroup::addOption(const SharedOptPtr& option) {
+	options_.push_back(option);
+}
 
 std::size_t OptionGroup::maxColumn(DescriptionLevel level) const {
 	std::size_t maxW = 0;
@@ -311,7 +311,9 @@ OptionContext::OptionContext(const std::string& cap, DescriptionLevel def)
 }
 OptionContext::~OptionContext()
 {}
-
+const std::string& OptionContext::caption() const {
+	return caption_;
+}
 void OptionContext::setActiveDescLevel(DescriptionLevel x) {
 	descLevel_ = std::min(x, desc_level_all);
 }
@@ -336,6 +338,14 @@ OptionContext& OptionContext::add(const OptionGroup& options) {
 	return *this;
 }
 
+OptionContext& OptionContext::addAlias(const std::string& aliasName, option_iterator option) {
+	if (option != end() && !aliasName.empty()) {
+		key_type k(option - begin());
+		if (!index_.insert(Name2Key::value_type(aliasName, k)).second) { throw DuplicateOption(caption(), aliasName); }
+	}
+	return *this;
+}
+
 const OptionGroup& OptionContext::findGroup(const std::string& name) const {
 	std::size_t x = findGroupKey(name);
 	if (x < groups_.size()) { return groups_[x]; }
@@ -374,49 +384,45 @@ void OptionContext::insertOption(size_t groupId, const SharedOptPtr& opt) {
 }
 
 OptionContext::option_iterator OptionContext::find(const char* key, FindType t) const {
-	PrefixRange r = findImpl(key, t);
-	if (std::distance(r.first, r.second) == 0) {
-		throw UnknownOption(caption(), (key ? key : ""));
-	}
-	if (std::distance(r.first, r.second) > 1) {
-		std::string str;
-		for (; r.first != r.second; ++r.first) { 
-			str += "  ";
-			str += r.first->first;
-			str += "\n";
-		}
-		throw AmbiguousOption(caption(), key, str);
-	}
-	return options_.begin() + r.first->second;
+	return options_.begin() + findImpl(key, t, unsigned(-1)).first->second;
 }
 
 OptionContext::option_iterator OptionContext::tryFind(const char* key, FindType t) const {
-	PrefixRange r = findImpl(key, t);
-	if (std::distance(r.first, r.second) == 0 || std::distance(r.first, r.second) > 1) {
-		return end();
-	}
-	return options_.begin() + r.first->second;
+	PrefixRange r = findImpl(key, t, 0u);
+	return std::distance(r.first, r.second) == 1 ? options_.begin() + r.first->second : end();
 }
 
-OptionContext::PrefixRange OptionContext::findImpl(const char* key, FindType t) const {
+OptionContext::PrefixRange OptionContext::findImpl(const char* key, FindType t, unsigned eMask, const std::string& eCtx) const {
 	std::string k(key ? key : "");
 	if (t == find_alias && !k.empty() && k[0] != '-') {
 		k   += k[0];
 		k[0] = '-';
 	}
-	index_iterator it   = index_.lower_bound(k);
+	index_iterator it = index_.lower_bound(k);
+	index_iterator up = it;
 	if (it != index_.end()) {
 		if ( (it->first == k) && ( (t & (find_alias|find_name)) != 0 ) ) {
-			return PrefixRange(it, ++index_iterator(it));
+			++up;
 		}
-		index_iterator up = it;
-		if ( (t & find_prefix) != 0 ) {
+		else if ( (t & find_prefix) != 0 ) {
 			k += char(CHAR_MAX);
 			up = index_.upper_bound(k);
+			k.erase(k.end()-1);
 		}
-		return PrefixRange(it, up);
 	}
-	return PrefixRange(index_.end(), index_.end()); 
+	if (std::distance(it, up) != 1 && eMask) {
+		if ((eMask & 1u) && it == up) { throw UnknownOption(eCtx, k); }
+		if ((eMask & 2u) && it != up) { 
+			std::string str;
+			for (; it != up; ++it) { 
+				str += "  ";
+				str += it->first;
+				str += "\n";
+			}
+			throw AmbiguousOption(eCtx, k, str);
+		}
+	}
+	return PrefixRange(it, up); 
 }
 
 OptionOutput& OptionContext::description(OptionOutput& out) const {
@@ -489,22 +495,27 @@ bool OptionContext::assignDefaults(const ParsedOptions& opts) const {
 ///////////////////////////////////////////////////////////////////////////////
 ParsedOptions::ParsedOptions()  {}
 ParsedOptions::~ParsedOptions() { parsed_.clear(); }
-bool ParsedOptions::assign(const ParsedValues& p) {
+bool ParsedOptions::assign(const ParsedValues& p, const ParsedOptions* exclude) {
 	if (!p.ctx) return false;
 	struct Assign {
-		Assign(ParsedOptions* x) : self(x) {}
+		Assign(ParsedOptions* x, const ParsedOptions* exclude) : self(x), ignore(exclude) {}
 		void assign(const ParsedValues& p) {
 			begin = it = p.begin();
 			// assign parsed values
 			for (ParsedValues::iterator end = p.end(); it != end; ++it) {
 				const Option& o = *it->first;
-				self->assign(o, it->second);
+				if (ignore && ignore->count(o.name()) != 0 && !o.value()->isComposing()){ 
+					continue;
+				}
+				if (int ret = self->assign(o, it->second)) {
+					throw ValueError(p.ctx ? p.ctx->caption() : "", static_cast<ValueError::Type>(ret-1), o.name(), it->second);
+				}
 			}
 		}
 		~Assign() { 
 			for (ParsedValues::iterator x = begin, end = this->it; x != end; ++x) {
 				const Option& o = *x->first;
-				assert(o.value()->state() == Value::value_fixed || self->parsed_.count(o.name()) != 0);
+				assert(o.value()->state() == Value::value_fixed || self->parsed_.count(o.name()) != 0 || ignore->count(o.name()) != 0);
 				if (o.value()->state() == Value::value_fixed) {
 					self->parsed_.insert(x->first->name());
 					o.value()->state(Value::value_unassigned);
@@ -512,23 +523,25 @@ bool ParsedOptions::assign(const ParsedValues& p) {
 			}
 		}
 		ParsedOptions* self;
+		const ParsedOptions*   ignore;
 		ParsedValues::iterator begin;
 		ParsedValues::iterator it;
-	} scoped(this);
+	} scoped(this, exclude);
 	scoped.assign(p);
 	return true;
 }
-void ParsedOptions::assign(const Option& o, const std::string& value) {
+int ParsedOptions::assign(const Option& o, const std::string& value) {
 	unsigned badState = 0;
 	if (!o.value()->isComposing()) {
-		if (parsed_.count(o.name())) { return ; }
+		if (parsed_.count(o.name())) { return 0; }
 		badState = (Value::value_fixed & o.value()->state());
 	}
 	if (badState || !o.value()->parse(o.name(), value, Value::value_fixed)) {
-		badState 
-			? throw ValueError("", ValueError::multiple_occurences, o.name(), value)
-			: throw ValueError("", ValueError::invalid_value, o.name(), value);
+		return badState 
+			? 1 + ValueError::multiple_occurences
+			: 1 + ValueError::invalid_value;
 	}
+	return 0;
 }
 ///////////////////////////////////////////////////////////////////////////////
 // class ParsedValues
@@ -550,63 +563,49 @@ void ParsedValues::add(const std::string& name, const std::string& value) {
 ///////////////////////////////////////////////////////////////////////////////
 // class OptionParser
 ///////////////////////////////////////////////////////////////////////////////
-OptionParser::OptionParser(const OptionContext& o, bool allowUnreg)
-	: pv_(o)
-	, allowUnreg_(allowUnreg)
+OptionParser::OptionParser(ParseContext& o)
+	: ctx_(&o)
 {}
 
 OptionParser::~OptionParser()
 {}
 
-ParsedValues OptionParser::parse() {
+ParseContext& OptionParser::parse() {
 	doParse();
-	return pv_;
-}
-SharedOptPtr OptionParser::tryGetOption(const char* name, FindType ft) const {
-	OptionContext::option_iterator it = pv_.ctx->tryFind(name, ft);
-	return it != pv_.ctx->end() ? *it : SharedOptPtr(0);
-}
-SharedOptPtr OptionParser::getOption(const char* name, FindType ft) const {
-	return !allowUnreg_ ? *pv_.ctx->find(name, ft) : tryGetOption(name, ft); 
+	return *ctx_;
 }
+ParseContext::~ParseContext() {}
 namespace {   
 ///////////////////////////////////////////////////////////////////////////////
 // class CommandLineParser
 ///////////////////////////////////////////////////////////////////////////////    
 class CommandLineParser : public OptionParser {
 public:
-	enum OptionType {short_opt, neg_opt, long_opt, end_opt, no_opt};
-	CommandLineParser(const OptionContext& o, bool allowUnreg, PosOption po)
-		: OptionParser(o, allowUnreg)
-		, posOpt_(po)
+	enum OptionType {short_opt, long_opt, end_opt, no_opt};
+	CommandLineParser(ParseContext& ctx, unsigned f)
+		: OptionParser(ctx)
+		, flags(f)
 	{}
 	std::vector<const char*> remaining;
+	unsigned flags;
 private:
 	virtual const char* next() = 0;
 	void doParse() {
 		bool breakEarly  = false;
-		SharedOptPtr po(0);
-		std::string poName, poNameLast;
+		int  posKey      = 0;
 		const char* curr;
-		while ( (curr=next()) && !breakEarly ) {
+		while ( (curr=next()) != 0 && !breakEarly ) {
 			switch(getOptionType(curr)) {
 				case short_opt: if (handleShortOpt(curr + 1)) curr = 0; break;
-				case neg_opt :  if (handleNegOpt(curr+5)) { curr = 0; break; } // else: fall through
-				case long_opt:  if (handleLongOpt(curr + 2)) curr = 0;  break;
+				case long_opt:  if (handleLongOpt(curr + 2))  curr = 0; break;
 				case end_opt:   curr = 0; breakEarly = true; break;
-				case no_opt:  
-				if (!posOpt_ || !posOpt_(curr, poName)) {
-					po = getOption("Positional Option", OptionContext::find_name_or_prefix);
-				}
-				else if (poName != poNameLast) {
-					po = getOption(poName.c_str(), OptionContext::find_name_or_prefix);
-					poNameLast = poName;
-				}
-				if (po.get())  {
-					addOptionValue(po, curr);
-					curr = 0;
-				}
-				break;
+				case no_opt: {
+					SharedOptPtr opt = getOption(posKey++, curr);
+					if (opt.get()) {
+						addOptionValue(opt, curr);
+						curr = 0;
+					}
+					break;}
 				default:
 					assert(0);
 			}
@@ -621,8 +620,7 @@ private:
 	}
 	OptionType getOptionType(const char* o) const {
 		if (strncmp(o, "--", 2) == 0) {
-			if (strncmp(o+2, "no-", 3) == 0) return neg_opt;
-			else                             return o[2] ? long_opt : end_opt;
+			return o[2] ? long_opt : end_opt;
 		}
 		return *o == '-' && *(o + 1) != '\0' ? short_opt : no_opt;
 	}
@@ -648,7 +646,7 @@ private:
 						++optName;
 					}
 				}
-				else if (*val != 0 || (val = next())) {
+				else if (*val != 0 || (val = next()) != 0) {
 					// -ovalue or -o value
 					addOptionValue(o, val);
 					return true;
@@ -663,14 +661,6 @@ private:
 		}
 		return true;
 	}
-	bool handleNegOpt(const char* optName) {
-		SharedOptPtr o = tryGetOption(optName, OptionContext::find_name_or_prefix);
-		if (o.get() && o->value()->isNegatable()) {
-			addOptionValue(o, "no");
-			return true;
-		}
-		return false;
-	}
 	bool handleLongOpt(const char* optName) {
 		string name(optName);
 		string value;
@@ -679,13 +669,28 @@ private:
 			value.assign(name, p + 1, string::npos);
 			name.erase(p, string::npos);
 		}
-		SharedOptPtr o;
-		if ( (o = getOption(name.c_str(), OptionContext::find_name_or_prefix)).get() ) {
+		SharedOptPtr o, on;
+		bool neg = false;
+		if (value.empty() && std::strncmp(optName, "no-", 3) == 0) {
+			try        { on = getOption(optName+3, OptionContext::find_name_or_prefix); }
+			catch(...) {}
+			if (on.get() && !on->value()->isNegatable()) { on.reset(); }
+		}
+		try  { o = getOption(name.c_str(), OptionContext::find_name_or_prefix); }
+		catch (const UnknownOption&) { 
+			if (!on.get()) { throw; } 
+		}
+		if (!o.get() && on.get()) {
+			std::swap(o, on);
+			value = "no";
+			neg   = true;
+		}
+		if (o.get()) {
 			if (!o->value()->isImplicit() && value.empty()) {
 				if (const char* v = next()) { value = v; }
 				else { throw SyntaxError(SyntaxError::missing_value, name); }
 			}
-			else if (o->value()->isFlag() && !value.empty()) {
+			else if (o->value()->isFlag() && !value.empty() && !neg && (flags & unsigned(command_line_allow_flag_value)) == 0u) {
 				// flags don't have values
 				throw SyntaxError(SyntaxError::extra_value, name);
 			}
@@ -694,14 +699,12 @@ private:
 		}
 		return false;
 	}
-	PosOption posOpt_;
-
 };
 
 class ArgvParser : public CommandLineParser {
 public:
-	ArgvParser(const OptionContext& o, bool allowUnreg, int startPos, char** argv, PosOption po)
-		: CommandLineParser(o, allowUnreg, po)
+	ArgvParser(ParseContext& ctx, int startPos, char** argv, unsigned flags)
+		: CommandLineParser(ctx, flags)
 		, currentArg_(0)
 		, argPos_(startPos)
 		, argv_(argv) {
@@ -719,29 +722,30 @@ private:
 
 class CommandStringParser : public CommandLineParser {
 public:
-	CommandStringParser(const std::string& cmd, const OptionContext& o, bool allowUnreg, PosOption po)
-		: CommandLineParser(o, allowUnreg, po)
-		, cmd_(cmd)
-		, pos_(0)
-		, end_(0) {
+	CommandStringParser(const char* cmd, ParseContext& ctx, unsigned flags)
+		: CommandLineParser(ctx, flags)
+		, cmd_(cmd ? cmd : "") {
+		tok_.reserve(80);
 	}
 private:
 	const char* next() {
-		const std::string::size_type eos = cmd_.size();
-		// skip any leading whitespace
-		while (end_ != eos && std::isspace(static_cast<unsigned char>(cmd_[end_])))  { ++end_; }
-		if (end_ == eos) { return 0; }
+		// skip leading white
+		while (std::isspace(static_cast<unsigned char>(*cmd_))) { ++cmd_; }
+		if (!*cmd_) return 0;
+		tok_.clear();
 		// find end of current arg
-		pos_ = end_++;
-		while (end_ != eos && !std::isspace(static_cast<unsigned char>(cmd_[end_]))) { ++end_; }
-		// terminate and return current arg
-		if (end_ != eos) { cmd_[end_++] = '\0'; }
-		return cmd_.c_str() + pos_;
+		for (char c, t = ' ', n; (c = *cmd_) != 0; ++cmd_) {
+			if      (c == t)                                      { if (t == ' ') break; t = ' '; }
+			else if ((c == '\'' || c == '"') && t == ' ')         { t = c; }
+			else if (c != '\\')                                   { tok_ += c; }
+			else if ((n=cmd_[1]) == '"' || n == '\'' || n == '\\'){ tok_ += n; ++cmd_; }
+			else                                                  { tok_ += c; }
+		}
+		return tok_.c_str();
 	}
 	CommandStringParser& operator=(const CommandStringParser&);
-	std::string             cmd_;
-	std::string::size_type  pos_;
-	std::string::size_type  end_;
+	const char* cmd_;
+	std::string tok_;
 };
 ///////////////////////////////////////////////////////////////////////////////
 // class CfgFileParser
@@ -749,11 +753,11 @@ private:
 class CfgFileParser : public OptionParser
 {
 public:
-	CfgFileParser(const OptionContext& o, bool allowUnreg, std::istream& in)
-		: OptionParser(o, allowUnreg)
+	CfgFileParser(ParseContext& ctx, std::istream& in)
+		: OptionParser(ctx)
 		, in_(in)
 	{}
-private:
+private: void operator=(const CfgFileParser&);
 	inline void trimLeft(std::string& str, const std::string& charList = " \t") {
 		std::string::size_type pos = str.find_first_not_of(charList);
 		if (pos != 0)
@@ -805,10 +809,8 @@ private:
 			if ( (pos = line.find("=")) != std::string::npos) {
 				// A new section terminates a multi line section value.
 				// First process the current section value...
-				if (inSection) {
-					if ( (opt = getOption(sectionName.c_str(), ft)).get() )
-						addOptionValue(opt, sectionValue);
-					inSection = false;
+				if (inSection && (opt = getOption(sectionName.c_str(), ft)).get()) {
+					addOptionValue(opt, sectionValue);
 				}
 				// ...then save the new section's value.
 				splitHalf(line, "=", sectionName, sectionValue);
@@ -831,28 +833,55 @@ private:
 	}
 	std::istream& in_;
 };
+class DefaultContext : public ParseContext {
+public:
+	DefaultContext(const OptionContext& o, bool allowUnreg, PosOption po)
+		: posOpt(po)
+		, parsed(o)
+		, eMask(2u + unsigned(!allowUnreg)) {}
+	SharedOptPtr  getOption(const char* name, FindType ft) {
+		OptionContext::OptionRange r = parsed.ctx->findImpl(name, ft, eMask);
+		if (r.first != r.second) { return *(parsed.ctx->begin() + r.first->second); }
+		return SharedOptPtr(0);
+	}
+	SharedOptPtr  getOption(int, const char* tok) {
+		std::string optName;
+		if (!posOpt || !posOpt(tok, optName)) { return getOption("Positional Option", OptionContext::find_name_or_prefix); }
+		return getOption(optName.c_str(), OptionContext::find_name_or_prefix);
+	}
+	void          addValue(const SharedOptPtr& key, const std::string& value) { parsed.add(key, value); }
+	PosOption    posOpt;
+	ParsedValues parsed;
+	unsigned     eMask;
+};
 
 } // end unnamed namespace
 
-ParsedValues parseCommandLine(int& argc, char** argv, const OptionContext& o, bool allowUnreg, PosOption po) {
-	ArgvParser pa(o, allowUnreg, 1, argv, po);
-	ParsedValues ret = pa.parse();
-	argc = 1 + (int)pa.remaining.size();
+ParsedValues parseCommandLine(int& argc, char** argv, const OptionContext& o, bool allowUnreg, PosOption po, unsigned flags) {
+	DefaultContext ctx(o, allowUnreg, po);
+	return static_cast<DefaultContext&>(parseCommandLine(argc, argv, ctx, flags)).parsed;
+}
+ParseContext& parseCommandLine(int& argc, char** argv, ParseContext& ctx, unsigned flags) {
+	ArgvParser parser(ctx, 1, argv, flags);
+	parser.parse();
+	argc = 1 + (int)parser.remaining.size();
 	for (int i = 1; i != argc; ++i) {
-		argv[i] = const_cast<char*>(pa.remaining[i-1]);
+		argv[i] = const_cast<char*>(parser.remaining[i-1]);
 	}
 	argv[argc] = 0;
-	return ret;
+	return ctx;
 }
-
-ParsedValues parseCommandString(const std::string& cmd, const OptionContext& o, bool allowUnreg, PosOption po) {
-	CommandStringParser parser(cmd, o, allowUnreg, po);
-	return parser.parse();
+ParseContext& parseCommandString(const char* cmd, ParseContext& ctx, unsigned flags) {
+	return CommandStringParser(cmd, ctx, flags).parse();
+}
+ParsedValues parseCommandString(const std::string& cmd, const OptionContext& o, bool allowUnreg, PosOption po, unsigned flags) {
+	DefaultContext ctx(o, allowUnreg, po);
+	return static_cast<DefaultContext&>(CommandStringParser(cmd.c_str(), ctx, flags).parse()).parsed;
 }
 
 ParsedValues parseCfgFile(std::istream& in, const OptionContext& o, bool allowUnreg) {
-	CfgFileParser p(o, allowUnreg, in);
-	return p.parse();
+	DefaultContext ctx(o, allowUnreg, 0);
+	return static_cast<DefaultContext&>(CfgFileParser(ctx, in).parse()).parsed;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
